/***************************************************************************
 * jtalk -- talk-like Jabber client
 * (C) Nicolas George - 2006
 * jtalk.h: common header
 ***************************************************************************/

/*
  This file is part of jtalk.

  jtalk is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  jtalk is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with jtalk; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdint.h>
#include <glib.h>
#include <loudmouth/loudmouth.h>

/* 1000 * unix time */
typedef uint_least64_t jtalk_timestamp_t;

typedef struct jtalk_connection_s jtalk_connection_t;

typedef struct jtalk_contact_s jtalk_contact_t;
typedef struct jtalk_message_s jtalk_message_t;

struct jtalk_connection_s {
    const char *server;
    const char *user;
    const char *pass;
    const char *resource;
    const char *jid;
    LmConnection *con;
    LmSSL *tls;
    LmMessageHandler *handler;
    GSList *pending_messages;
    int port;
    int reconnect_delay;
    unsigned connected : 1;
};

struct jtalk_contact_s {
    char *jid;
    char *short_name;
    GList *msg_first, *msg_last;
    GSList *available_res;
    jtalk_timestamp_t available_ts;
    char *log_file_name;
    int undelivered_msg;
    unsigned subscribed_from : 1, subscribed_to : 1;
};

struct jtalk_message_s {
    jtalk_message_t *all_next, *all_prev;
    jtalk_timestamp_t timestamp;
    jtalk_contact_t *from;
    GSList *to;
    jtalk_connection_t *connection;
    char *resource;
    unsigned delivered : 1;
    char text[0];
};

typedef struct {
    unsigned char *buffer;
    size_t buffer_size, buffer_have;
    jtalk_connection_t *preferred_connection;
    int fd;
    guint source_id;
} jtalk_client_t;

/* main.c */
extern void jtalk_print_timestamp(jtalk_timestamp_t, char *);
extern jtalk_timestamp_t jtalk_now(void);
extern void jtalk_client_init(jtalk_client_t *);
extern void jtalk_client_printf(jtalk_client_t *, const char *, ...);
extern void jtalk_line_from_client(jtalk_client_t *, unsigned char *);
extern void jtalk_got_chat_text(jtalk_connection_t *, const char *,
    const char *);
extern void jtalk_command_from_client(jtalk_client_t *, unsigned char *);
extern void jtalk_client_closed(jtalk_client_t *);
extern void jtalk_got_roster_contact(const char *, int, int);

/* loudmouth.c */
extern jtalk_connection_t *jtalk_connection_new(char **, int);
extern void jtalk_connection_send_chat_message(jtalk_connection_t *,
    jtalk_contact_t *, char *);

/* socket.c */
extern void socket_server_start(void);
extern void socket_write(jtalk_client_t *, const char *);

/* settings.c */
extern const char *jtalk_user;
extern const char *jtalk_esc_reset;
extern const char *jtalk_esc_error;
extern const char *jtalk_esc_name;
extern const char *jtalk_esc_self_name;
extern const char *jtalk_esc_online;
extern const char *jtalk_esc_offline;
extern const char *jtalk_esc_undelivered;
extern const char *jtalk_esc_delivered;
extern size_t jtalk_opt_max_history;
extern jtalk_timestamp_t jtalk_opt_deliver_old_messages;
extern int jtalk_opt_connection_choose_connected;
extern int jtalk_opt_connection_choose_messages;
extern int jtalk_opt_connection_choose_undelivered;
extern void jtalk_settings_init(void);
extern char *jtalk_settings_get_file(const char *);

/* terminal.c */
extern void jtalk_tell_user_direct_and_free(int, char *);
extern void jtalk_tell_user(int, const char *, ...);

/* contacts.c */
extern void jtalk_contact_init(void);
extern jtalk_contact_t *jtalk_contact_find(const char *, int);
extern GList *jtalk_contact_get_all(void);
extern jtalk_message_t *jtalk_message_new(const char *, const char *);
extern void jtalk_contact_add_message(jtalk_contact_t *, jtalk_message_t *);
extern void jtalk_contact_set_available(jtalk_contact_t *, const char *, int);
extern GList *jtalk_contact_sort(GList *);

/* log.c */
extern char *jtalk_log_file_name(jtalk_contact_t *);
extern void jtalk_log_message(jtalk_message_t *);

/* sorter.c */
extern void jtalk_sorter_command(jtalk_client_t *, char **, int);
extern int jtalk_sorter_eval_v(int, char *(*)[2], int);
extern int jtalk_sorter_eval_l(int, ...);
