/***************************************************************************
 * jtalk -- talk-like Jabber client
 * (C) Nicolas George - 2006
 * terminal.c: interface with the user terminal and idle handling
 ***************************************************************************/

/*
  This file is part of jtalk.

  jtalk is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  jtalk is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with jtalk; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <utmpx.h>
#include <glib.h>

#include "jtalk.h"

static FILE *
find_best_tty(void)
{
    struct utmpx *u;
    struct stat st;
    char dev[6 + sizeof(u->ut_line)];
    char bestdev[6 + sizeof(u->ut_line)] = "";
    time_t bestdev_time = 0;
    FILE *f;

    setutxent();
    while((u = getutxent()) != NULL) {
	if(strcmp(u->ut_user, jtalk_user) != 0)
	    continue;
	snprintf(dev, sizeof(dev), "/dev/%s", u->ut_line);
	if(stat(dev, &st) < 0)
	    continue;
	if((st.st_mode & 0220) != 0220)
	    continue;
	if(st.st_atime > bestdev_time) {
	    bestdev_time = st.st_atime;
	    memcpy(bestdev, dev, sizeof(dev));
	}
    }
    if(bestdev[0] == 0) {
	abort(); /* TODO */
    }
    f = fopen(bestdev, "w");
    return(f);
}

void
jtalk_tell_user_direct_and_free(int bell, char *t)
{
    FILE *tty;

    tty = find_best_tty();
    fprintf(tty, "%sjtalk: %s\n", bell ? "\a" : "", t);
    fclose(tty);
    g_free(t);
}

void
jtalk_tell_user(int value, const char *format, ...)
{
    va_list args;
    char *t;

    if(value < 0)
	return;
    va_start(args, format);
    t = g_strdup_vprintf(format, args);
    va_end(args);
    jtalk_tell_user_direct_and_free(value >= 10, t);
}
